︠1621e47b-3836-494f-846b-0ec326c7d106is︠
%hide
%auto

def transformation_matrix(M):
    columns = []
    for evalue, evecs, mult in M.eigenvectors_right():
        if evalue.imag() < 0:
            continue
        if evalue.imag():
            for evec in evecs:
                u = vector(RDF, [a.real() for a in evec])
                v = vector(RDF, [a.imag() for a in evec])
                columns.append(u)
                columns.append(v)
        else:
            for v in evecs:
                columns.append(vector(RDF, v))
    return column_matrix(columns)


def axes3d(x_range, y_range, z_range, **options):
    x0, y0, z0 = options.pop("origin", (0, 0, 0))
    options.setdefault("color", "black")
    x, xmin, xmax = x_range
    y, ymin, ymax = y_range
    z, zmin, zmax = z_range
    xmin, xmax = min(x0, xmin), max(x0, xmax)
    ymin, ymax = min(y0, ymin), max(y0, ymax)
    zmin, zmax = min(z0, zmin), max(z0, zmax)
    p = Graphics()
    p += arrow3d((xmin, y0, z0), (xmax, y0, z0), width=1, **options)
    p += arrow3d((x0, ymin, z0), (x0, ymax, z0), width=1, **options)
    p += arrow3d((x0, y0, zmin), (x0, y0, zmax), width=1, **options)
    p += text3d(str(x), (xmax + (xmax - xmin)*0.05, y0, z0), **options)
    p += text3d(str(y), (x0, ymax + (ymax - ymin)*0.05, z0), **options)
    p += text3d(str(z), (x0, y0, zmax + (zmax - zmin)*0.05), **options)
    return p


x, y, z = var("x, y, z")
def plot_eigenspaces3d(M, init_conds, vars=(x, y, z), min=-5, max=5):
    x, y, z = vars
    u, v = var("u, v")
    p = axes3d((x, min, max), (y, min, max), (z, min, max), grid=False, color="gray")

    for init_cond, n_max in init_conds.items():
        solution = [vector(init_cond)]
        for n in xrange(n_max):
            solution.append(M*solution[-1])
        p += list_plot(solution, color="blue", size=8)
        p += list_plot(solution, color="blue", plotjoined=True)

    for evalue, evecs, mult in M.eigenvectors_right():
        if evalue.imag() < 0:
            continue # For a complex eigenvalue, only use one of the conjugate pair
        if abs(evalue) < 1:
            color="green"
        elif abs(evalue) > 1:
            color="coral"
        else:
            color="orange"
        if evalue.imag():
            # Draw the plane spanned by the real and imaginary parts of a complex eigenvector
            realvec = vector(RDF, [a.real() for a in evecs[0]])
            imagvec = vector(RDF, [a.imag() for a in evecs[0]])
            u1 = realvec.normalized()
            u2 = (imagvec - imagvec.dot_product(u1)*u1).normalized()
            p += parametric_plot3d([a*u + b*v for a, b in zip(u1, u2)],
                (u, min, max), (v, min, max), color="light"+color, opacity=0.7)
        elif len(evecs) > 1:
            # Draw the plane spanned by two eigenvectors
            evecs = [vector(RDF, vec) for vec in evecs] # Partly a hack due to a bug in Sage
            u1 = evecs[0].normalized()
            u2 = (evecs[1] - evecs[1].dot_product(u1)*u1).normalized()
            p += parametric_plot3d([a*u + b*v for a, b in zip(u1, u2)],
                (u, min, max), (v, min, max), color="light"+color, opacity=0.7)
        else:
            # Draw the line spanned by a single eigenvector
            u1 = evecs[0].normalized()
            p += line3d((min*u1, max*u1), color=color, thickness=4, opacity=1)

    return p
︡30b89f8c-e9ee-4d5d-843d-9deea50362e2︡{"hide":"input"}︡{"done":true}
︠a189f0be-508a-41e3-bad9-bbd0aa484b8ei︠
%hide
%md
# Recall the black bear population model:
$$ \begin{bmatrix} J_{t+1} \\ A_{t+1} \end{bmatrix} = \begin{bmatrix} 0.65 & 0.5 \\ 0.25 & 0.9 \end{bmatrix} \begin{bmatrix} J_t \\ A_t \end{bmatrix} $$

The interactive below shows its behavior. As usual, to use interactives in this worksheet, click “Open in CoCalc” at the top right of this page.
︡57870e9b-a404-4c1e-a0b1-71a98e232ff7︡{"hide":"input"}︡{"md":"# Recall the black bear population model:\n$$ \\begin{bmatrix} J_{t+1} \\\\ A_{t+1} \\end{bmatrix} = \\begin{bmatrix} 0.65 & 0.5 \\\\ 0.25 & 0.9 \\end{bmatrix} \\begin{bmatrix} J_t \\\\ A_t \\end{bmatrix} $$\n\nThe interactive below shows its behavior. As usual, to use interactives in this worksheet, click “Open in CoCalc” at the top right of this page. "}︡{"done":true}︡
︠a083943f-a6dc-4033-8f3f-c6fe31d723b5s︠
@interact
def discretetime(iterations=(0, 20, 1),
                 plotjoined=checkbox(True, label="Connect dots"),
                 show_eigenvectors=checkbox(False, label="Show eigenvector line(s)")):
    M = matrix(RDF, [
        [0.65, 0.5],
        [0.25, 0.9],
    ])
    ics = {
        (2, 5): "blue",
        (7, 4): "purple",
        (0, 3): "gold",
        (9, 0): "fuchsia",
    }
    p = Graphics()
    if show_eigenvectors:
        for eval, evec, mult in M.eigenvectors_right():
            if imag(eval):
                continue
            slope = evec[0][1]/evec[0][0]
            color = "green" if abs(eval) < 1 else "red"
            p += plot(slope*x, (x, -20, 20), color=color, thickness=2)
    for state, color in ics.items():
        states = [vector(state)]
        for i in xrange(iterations):
            states.append(M*states[-1])
        p += list_plot(states, plotjoined=False, color=color, size=50)
        if plotjoined:
            p += list_plot(states, plotjoined=True, color=color)
    show(p, xmin=-5, ymin=-5, xmax=20, ymax=20, aspect_ratio=1, axes_labels=("$J$", "$A$"))
︡048b2443-f572-4f9a-b839-cfee2d6d20f0︡{"interact":{"controls":[{"animate":true,"control_type":"slider","default":0,"display_value":true,"label":"iterations","vals":["0","1","2","3","4","5","6","7","8","9","10","11","12","13","14","15","16","17","18","19","20"],"var":"iterations","width":null},{"control_type":"checkbox","default":true,"label":"Connect dots","readonly":false,"var":"plotjoined"},{"control_type":"checkbox","default":false,"label":"Show eigenvector line(s)","readonly":false,"var":"show_eigenvectors"}],"flicker":false,"id":"c03d7730-9022-42a9-95d4-0f0f751077f4","layout":[[["iterations",12,null]],[["plotjoined",12,null]],[["show_eigenvectors",12,null]],[["",12,null]]],"style":"None"}}︡{"done":true}
︠c34da4bd-27da-46f3-a847-44b0834d76dci︠
%hide
%md
### Below is another interactive showing what happens to states along the eigenvector lines, including states in the other three quadrants. Even though this is biologically meaningless, it will help us to understand the overall behavior of the system.
︡a86a3446-9ea8-4b35-8e87-01de15221367︡{"hide":"input"}︡{"md":"### Below is another interactive showing what happens to states along the eigenvector lines, including states in the other three quadrants. Even though this is biologically meaningless, it will help us to understand the overall behavior of the system. "}︡{"done":true}︡
︠f171cb2b-2d2e-485f-bd2d-48f75fcce59b︠
@interact
def discretetime(iterations=(0, 20, 1),
                 plotjoined=checkbox(True, label="Connect dots"),
                 show_eigenvectors=checkbox(False, label="Show eigenvector line(s)")):
    M = matrix(RDF, [
        [0.65, 0.5],
        [0.25, 0.9],
    ])
    ics = {
        (1, 1): "darkred",
        (-1, -1): "darkred",
        (-20, 10): "darkgreen",
        (20, -10): "darkgreen",
    }
    p = Graphics()
    if show_eigenvectors:
        for eval, evec, mult in M.eigenvectors_right():
            if imag(eval):
                continue
            slope = evec[0][1]/evec[0][0]
            color = "green" if abs(eval) < 1 else "red"
            p += plot(slope*x, (x, -20, 20), color=color, thickness=2)
    for state, color in ics.items():
        states = [vector(state)]
        for i in xrange(iterations):
            states.append(M*states[-1])
        p += list_plot(states, plotjoined=False, color=color, size=50)
        if plotjoined:
            p += list_plot(states, plotjoined=True, color=color)
    show(p, xmin=-20, ymin=-20, xmax=20, ymax=20, aspect_ratio=1, axes_labels=("$J$", "$A$"))
︡04a5f02c-91fa-42cf-9329-465051167078︡{"interact":{"controls":[{"animate":true,"control_type":"slider","default":0,"display_value":true,"label":"iterations","vals":["0","1","2","3","4","5","6","7","8","9","10","11","12","13","14","15","16","17","18","19","20"],"var":"iterations","width":null},{"control_type":"checkbox","default":true,"label":"Connect dots","readonly":false,"var":"plotjoined"},{"control_type":"checkbox","default":false,"label":"Show eigenvector line(s)","readonly":false,"var":"show_eigenvectors"}],"flicker":false,"id":"71e2d583-907d-4aa1-ae40-72688b58e46f","layout":[[["iterations",12,null]],[["plotjoined",12,null]],[["show_eigenvectors",12,null]],[["",12,null]]],"style":"None"}}︡{"done":true}︡
︠40c57e44-79be-4f69-9585-4104dc6fa7a5i︠
%hide
%md
# Diagonal matrices:
$$ \begin{bmatrix} \lambda_1 & 0 & 0 \\ 0 & \lambda_2 & 0 \\ 0 & 0 & \lambda_3 \end{bmatrix}$$
︡bb8eb3b6-bbb7-4550-b78d-6c9f9612cacd︡{"hide":"input"}︡{"md":"# Diagonal matrices:\n$$ \\begin{bmatrix} \\lambda_1 & 0 & 0 \\\\ 0 & \\lambda_2 & 0 \\\\ 0 & 0 & \\lambda_3 \\end{bmatrix}$$"}︡{"done":true}︡
︠c7616ee1-63c6-436b-b847-b377e37eedbei︠
%hide
%md
### Growth along one axis, decay along the other:

First look at what happens just along the axes.
︡7f1046cd-dde1-4cee-a8ee-438be586bb9d︡{"hide":"input"}︡{"md":"### Growth along one axis, decay along the other:\n\nFirst look at what happens just along the axes. "}︡{"done":true}︡
︠cd94295e-1491-43bd-a225-2083300b12bc︠
D = diagonal_matrix(RDF, [1.2, 0.7])
size = 5
@interact
def discretetime(iterations=(0, 20, 1),
        plotjoined=checkbox(True, label="Connect dots"),
    ):
    ics = {
        (0, 5): "black",
        (1, 0): "black",
    }
    p = Graphics()
    for state, color in ics.items():
        states = [vector(state)]
        for i in xrange(iterations):
            states.append(D*states[-1])
        p += list_plot(states, plotjoined=False, color=color, size=50)
        if plotjoined:
            p += list_plot(states, plotjoined=True, color=color)
    show(D)
    p.show(xmin=-size, ymin=-size, xmax=size, ymax=size, aspect_ratio=1, axes_labels=("$X$", "$Y$"))
︡b6382642-5c01-493d-8bb5-8a46f4157f30︡{"interact":{"controls":[{"animate":true,"control_type":"slider","default":0,"display_value":true,"label":"iterations","vals":["0","1","2","3","4","5","6","7","8","9","10","11","12","13","14","15","16","17","18","19","20"],"var":"iterations","width":null},{"control_type":"checkbox","default":true,"label":"Connect dots","readonly":false,"var":"plotjoined"}],"flicker":false,"id":"6e1677a1-2cf0-44b0-a6bd-3167cb4d171e","layout":[[["iterations",12,null]],[["plotjoined",12,null]],[["",12,null]]],"style":"None"}}︡{"done":true}︡
︠84848604-d36e-4cc9-a6b3-66d7b9db854ai︠
%hide
%md
### Now, see what happens when you put both together.
︡645f32ef-b198-4f62-96fe-4e5e1bc78d5d︡{"hide":"input"}︡{"md":"### Now, see what happens when you put both together. "}︡{"done":true}︡
︠56ac1fa3-7fe1-4c94-a73b-d0a93d17285a︠
D = diagonal_matrix(RDF, [1.2, 0.7])
size = 5
@interact
def discretetime(iterations=(0, 20, 1),
        plotjoined=checkbox(True, label="Connect dots"),
    ):
    ics = {
        (0, 5): "black",
        (1, 0): "black",
        (1, 5): "black",
        (-1, 5): "black",
        (-1, -5): "black",
        (1, -5): "black",
    }
    p = Graphics()
    for state, color in ics.items():
        states = [vector(state)]
        for i in xrange(iterations):
            states.append(D*states[-1])
        p += list_plot(states, plotjoined=False, color=color, size=50)
        if plotjoined:
            p += list_plot(states, plotjoined=True, color=color)
    show(D)
    p.show(xmin=-size, ymin=-size, xmax=size, ymax=size, aspect_ratio=1, axes_labels=("$X$", "$Y$"))
︡ccf423a7-5630-477b-b86e-b2e0cab5ba5e︡{"interact":{"controls":[{"animate":true,"control_type":"slider","default":0,"display_value":true,"label":"iterations","vals":["0","1","2","3","4","5","6","7","8","9","10","11","12","13","14","15","16","17","18","19","20"],"var":"iterations","width":null},{"control_type":"checkbox","default":true,"label":"Connect dots","readonly":false,"var":"plotjoined"}],"flicker":false,"id":"64c1e95e-2279-4c4f-bd4c-2901ec079c05","layout":[[["iterations",12,null]],[["plotjoined",12,null]],[["",12,null]]],"style":"None"}}︡{"done":true}︡
︠2e5644ca-29d4-42a9-8537-f9e5f51cc32bi︠
%hide
%md
Note that the behavior above shows that the equilibrium point at the origin is a saddle point.

Note also that the long-term behavior is growth along the $X$ axis, because that is the axis that has exponential growth, caused by the larger eigenvalue 1.2.
︡aba6dfba-43ee-4279-9596-709e96aaa7eb︡{"hide":"input"}︡{"md":"Note that the behavior above shows that the equilibrium point at the origin is a saddle point. \n\nNote also that the long-term behavior is growth along the $X$ axis, because that is the axis that has exponential growth, caused by the larger eigenvalue 1.2. "}︡{"done":true}︡
︠c08aad83-edec-44c4-9428-db9aaed82c90i︠
%hide
%md
### Growth along both axes:
︡14339aac-b7ac-4662-b29b-ceeee09d3f24︡{"hide":"input"}︡{"md":"### Growth along both axes:"}︡{"done":true}︡
︠8766ae7a-b8eb-4d6a-9341-2f5c886b2ece︠
D = diagonal_matrix(RDF, [1.1, 1.3])
size = 8
@interact
def discretetime(iterations=(0, 20, 1),
        plotjoined=checkbox(True, label="Connect dots"),
    ):
    ics = {
        (0, 1): "black",
        (1, 0): "black",
    }
    p = Graphics()
    for state, color in ics.items():
        states = [vector(state)]
        for i in xrange(iterations):
            states.append(D*states[-1])
        p += list_plot(states, plotjoined=False, color=color, size=50)
        if plotjoined:
            p += list_plot(states, plotjoined=True, color=color)
    show(D)
    p.show(xmin=-size, ymin=-size, xmax=size, ymax=size, aspect_ratio=1, axes_labels=("$X$", "$Y$"))
︡58907403-6a0c-468f-9354-fabc3e93d389︡{"interact":{"controls":[{"animate":true,"control_type":"slider","default":0,"display_value":true,"label":"iterations","vals":["0","1","2","3","4","5","6","7","8","9","10","11","12","13","14","15","16","17","18","19","20"],"var":"iterations","width":null},{"control_type":"checkbox","default":true,"label":"Connect dots","readonly":false,"var":"plotjoined"}],"flicker":false,"id":"ea24d9b3-d16a-4cca-aa47-65c6f5a092d5","layout":[[["iterations",12,null]],[["plotjoined",12,null]],[["",12,null]]],"style":"None"}}︡{"done":true}︡
︠b83a0ac3-9820-4835-8863-7e89b628a6c0︠
D = diagonal_matrix(RDF, [1.1, 1.3])
size = 8
@interact
def discretetime(iterations=(0, 20, 1),
        plotjoined=checkbox(True, label="Connect dots"),
    ):
    ics = {
        (0, 1): "black",
        (1, 0): "black",
        (1, 1): "black",
        (-1, 1): "black",
        (-1, -1): "black",
        (1, -1): "black",
    }
    p = Graphics()
    for state, color in ics.items():
        states = [vector(state)]
        for i in xrange(iterations):
            states.append(D*states[-1])
        p += list_plot(states, plotjoined=False, color=color, size=50)
        if plotjoined:
            p += list_plot(states, plotjoined=True, color=color)
    show(D)
    p.show(xmin=-size, ymin=-size, xmax=size, ymax=size, aspect_ratio=1, axes_labels=("$X$", "$Y$"))
︡51aa563c-678b-4e85-9184-6cc55592e86d︡{"interact":{"controls":[{"animate":true,"control_type":"slider","default":0,"display_value":true,"label":"iterations","vals":["0","1","2","3","4","5","6","7","8","9","10","11","12","13","14","15","16","17","18","19","20"],"var":"iterations","width":null},{"control_type":"checkbox","default":true,"label":"Connect dots","readonly":false,"var":"plotjoined"}],"flicker":false,"id":"01f6368e-b5cb-41e1-ad21-61afe50577ba","layout":[[["iterations",12,null]],[["plotjoined",12,null]],[["",12,null]]],"style":"None"}}︡{"done":true}︡
︠dfe28adb-4c82-420f-b925-10a0bca75c81i︠
%hide
%md
In the case above, we see that the equilibrium point at the origin is unstable (a source). This should not be surprising at all.

More importantly, we also see that, although both the $X$ and $Y$ are increasing (and will increase exponentially forever), the long-term behavior is primarily growth in the $Y$ direction, because this axis has the *faster* exponential growth rate. Again, that faster growth rate is due to the larger of the two eigenvalues, 1.3.
︡3cccc70a-825d-48db-b040-5b8c12665496︡{"hide":"input"}︡{"md":"In the case above, we see that the equilibrium point at the origin is unstable (a source). This should not be surprising at all. \n\nMore importantly, we also see that, although both the $X$ and $Y$ are increasing (and will increase exponentially forever), the long-term behavior is primarily growth in the $Y$ direction, because this axis has the *faster* exponential growth rate. Again, that faster growth rate is due to the larger of the two eigenvalues, 1.3. "}︡{"done":true}︡
︠cf76b007-43a9-4733-9933-03746882a264i︠
%hide
%md
### Decay along both axes
︡89f0ddbe-4f8e-4042-9c26-c949e94c96d9︡{"hide":"input"}︡{"md":"### Decay along both axes"}︡{"done":true}︡
︠12b06fdd-08ec-4ae5-8075-bf67e93e02a5︠
D = diagonal_matrix(RDF, [0.9, 0.6])
size = 5
@interact
def discretetime(iterations=(0, 20, 1),
        plotjoined=checkbox(True, label="Connect dots"),
    ):
    ics = {
        (0, 5): "black",
        (5, 0): "black",
    }
    p = Graphics()
    for state, color in ics.items():
        states = [vector(state)]
        for i in xrange(iterations):
            states.append(D*states[-1])
        p += list_plot(states, plotjoined=False, color=color, size=50)
        if plotjoined:
            p += list_plot(states, plotjoined=True, color=color)
    show(D)
    p.show(xmin=-size, ymin=-size, xmax=size, ymax=size, aspect_ratio=1, axes_labels=("$X$", "$Y$"))
︡837a55cf-e886-48df-b0c3-61bd5ffba0d7︡{"interact":{"controls":[{"animate":true,"control_type":"slider","default":0,"display_value":true,"label":"iterations","vals":["0","1","2","3","4","5","6","7","8","9","10","11","12","13","14","15","16","17","18","19","20"],"var":"iterations","width":null},{"control_type":"checkbox","default":true,"label":"Connect dots","readonly":false,"var":"plotjoined"}],"flicker":false,"id":"b73d198d-0465-496a-8802-698818576f7d","layout":[[["iterations",12,null]],[["plotjoined",12,null]],[["",12,null]]],"style":"None"}}︡{"done":true}︡
︠5c257bc4-7665-4652-a5bf-7356eb804c2a︠
D = diagonal_matrix(RDF, [0.9, 0.6])
size = 5
@interact
def discretetime(iterations=(0, 20, 1),
        plotjoined=checkbox(True, label="Connect dots"),
    ):
    ics = {
        (0, 5): "black",
        (5, 0): "black",
        (5, 5): "black",
        (-5, 5): "black",
        (-5, -5): "black",
        (5, -5): "black",
    }
    p = Graphics()
    for state, color in ics.items():
        states = [vector(state)]
        for i in xrange(iterations):
            states.append(D*states[-1])
        p += list_plot(states, plotjoined=False, color=color, size=50)
        if plotjoined:
            p += list_plot(states, plotjoined=True, color=color)
    show(D)
    p.show(xmin=-size, ymin=-size, xmax=size, ymax=size, aspect_ratio=1, axes_labels=("$X$", "$Y$"))
︡c1802a23-5f3e-46fd-ac7f-2b477f630355︡{"interact":{"controls":[{"animate":true,"control_type":"slider","default":0,"display_value":true,"label":"iterations","vals":["0","1","2","3","4","5","6","7","8","9","10","11","12","13","14","15","16","17","18","19","20"],"var":"iterations","width":null},{"control_type":"checkbox","default":true,"label":"Connect dots","readonly":false,"var":"plotjoined"}],"flicker":false,"id":"c4ec9918-4153-4e98-b8a9-085eb974cb10","layout":[[["iterations",12,null]],[["plotjoined",12,null]],[["",12,null]]],"style":"None"}}︡{"done":true}︡
︠e441e9bd-48b5-4928-b148-f9660d570fc8i︠
%hide
%md
Again, it should not be surprising that the above equilibrium point is stable (a sink).

More importantly, in the above case we see that, although both $X$ and $Y$ are decaying exponentially, the trajectories always eventually approach the origin (equilibrium point) along the $X$ axis. So the long-term behavior is exponential decay (at 10% per iteration) along the $X$ axis. This time, this is caused by the fact that the $X$ axis has the *slower* of the two *decay* rates. But the slower of the two decay rates still means the *larger* of the two eigenvalues!
︡2dbf25d8-90c0-40df-b727-c551efed5d1c︡{"hide":"input"}︡{"md":"Again, it should not be surprising that the above equilibrium point is stable (a sink). \n\nMore importantly, in the above case we see that, although both $X$ and $Y$ are decaying exponentially, the trajectories always eventually approach the origin (equilibrium point) along the $X$ axis. So the long-term behavior is exponential decay (at 10% per iteration) along the $X$ axis. This time, this is caused by the fact that the $X$ axis has the *slower* of the two *decay* rates. But the slower of the two decay rates still means the *larger* of the two eigenvalues! "}︡{"done":true}︡
︠1e63ec48-c0ad-4add-aefc-1a2fcc9afe41i︠
%hide
%md
### Important conclusion!

In all of the cases above, the conclusion was that the *larger of the two eigenvalues* determined the long-term behavior of the system. This largest eigenvalue is called the dominant eigenvalue. Actually, since the exponential growth/decay rates are determined by the *absolute value* of the corresponding eigenvalue, ***the dominant eigenvalue is the one whose abolute value is largest***. To be more specific about its meaning:
* This dominant eigenvalue determines the long-term growth (or decay) rate of the system, *and*
* That growing (or decaying) in the long run will be primarily in the direction of (i.e. parallel to) the axis that corresponds to that eigenvalue.
︡1af590ac-9eb4-44a0-850f-7d46850e8f75︡{"hide":"input"}︡{"md":"### Important conclusion! \n\nIn all of the cases above, the conclusion was that the *larger of the two eigenvalues* determined the long-term behavior of the system. This largest eigenvalue is called the dominant eigenvalue. Actually, since the exponential growth/decay rates are determined by the *absolute value* of the corresponding eigenvalue, ***the dominant eigenvalue is the one whose abolute value is largest***. To be more specific about its meaning: \n* This dominant eigenvalue determines the long-term growth (or decay) rate of the system, *and* \n* That growing (or decaying) in the long run will be primarily in the direction of (i.e. parallel to) the axis that corresponds to that eigenvalue. "}︡{"done":true}︡
︠6072d077-088a-4974-bd91-b490c4e9a18e︠










︡4196e07e-3cd8-4962-9ad1-f2d02ae6d826︡
︠bac34980-e4cb-4ca5-9d26-06f6f5d1e3d3i︠
%hide
%md
# Coordinates

### First, standard $X, Y$ coordinates
︡a797d9d6-2408-47dc-a5f9-cc317ecebce1︡{"hide":"input"}︡{"md":"# Coordinates\n\n### First, standard $X, Y$ coordinates"}︡{"done":true}︡
︠02b66723-1caa-4cee-bf1c-30573f2a3f83︠
e1 = vector([1, 0])
e2 = vector([0, 1])
color1 = colors["blue"].darker(0.2)
color2 = colors["gold"].darker(0.2)
size = 5
basisvectors = plot(e1, color=color1) + plot(e2, color=color2)
old_grid = Graphics()
for c in xrange(-size + 1, size):
    old_grid += line(((-size, c), (size, c)), color=colors["white"].darker(0.1))
    old_grid += line(((c, -size), (c, size)), color=colors["white"].darker(0.1))
@interact
def oldcoordinates(x=(-size, size, 1), y=(-size, size, 1),
        show_old_grid=checkbox(True, label=u"Original x-y \u201ccoordinate grid\u201d"),
        show_vectors=checkbox(True, label="Standard basis vectors"),
        show_solution=checkbox(False, label="Solution to point along standard basis vectors"),
    ):
    p = Graphics()
    if show_old_grid:
        p += old_grid
    if show_vectors:
        p += basisvectors
    if show_solution:
        p += plot(x*e1, color=color1.darker(0.4))
        p += plot(y*e2, color=color2.darker(0.4), start=x*e1)
        print("x = {}, y = {}. That is, ({}, {}) = {} e1  +  {} e2".format(x, y, x, y, x, y))
    p += point((x, y), color="black", size=40)
    show(p, xmin=-size, xmax=size, ymin=-size, ymax=size, aspect_ratio=1, figsize=6, axes_labels=("$X$", "$Y$"))
︡56b62675-7bb0-405c-8935-9b91d95836ad︡{"interact":{"controls":[{"animate":true,"control_type":"slider","default":0,"display_value":true,"label":"x","vals":["-5","-4","-3","-2","-1","0","1","2","3","4","5"],"var":"x","width":null},{"animate":true,"control_type":"slider","default":0,"display_value":true,"label":"y","vals":["-5","-4","-3","-2","-1","0","1","2","3","4","5"],"var":"y","width":null},{"control_type":"checkbox","default":true,"label":"Original x-y “coordinate grid”","readonly":false,"var":"show_old_grid"},{"control_type":"checkbox","default":true,"label":"Standard basis vectors","readonly":false,"var":"show_vectors"},{"control_type":"checkbox","default":false,"label":"Solution to point along standard basis vectors","readonly":false,"var":"show_solution"}],"flicker":false,"id":"b3e198ca-a713-49b7-9287-338ca9f5c515","layout":[[["x",12,null]],[["y",12,null]],[["show_old_grid",12,null]],[["show_vectors",12,null]],[["show_solution",12,null]],[["",12,null]]],"style":"None"}}︡{"done":true}︡
︠a1f704be-437c-4f45-97ab-e538f6b54698i︠
%hide
%md
### Coordinates with respect to a different basis:

$$ \vec{u}_1 = \begin{bmatrix} 1 \\ 1 \end{bmatrix} , \qquad \vec{u}_2 = \begin{bmatrix} -2 \\ 1 \end{bmatrix} $$
︡a6c9e992-2313-4c02-8a2a-501181689680︡{"hide":"input"}︡{"md":"### Coordinates with respect to a different basis:\n\n$$ \\vec{u}_1 = \\begin{bmatrix} 1 \\\\ 1 \\end{bmatrix} , \\qquad \\vec{u}_2 = \\begin{bmatrix} -2 \\\\ 1 \\end{bmatrix} $$"}︡{"done":true}︡
︠43d1fc00-55cd-4d11-a6d4-627c297d29d6︠
u1 = vector([1, 1])
u2 = vector([-2, 1])
T = column_matrix([u1, u2])
color1 = colors["red"].darker(0.2)
color2 = colors["green"].darker(0.2)
size = 8
eigenvectors = plot(u1, color=color1) + plot(u2, color=color2)
old_grid = Graphics()
for c in xrange(-size + 1, size):
    old_grid += line(((-size, c), (size, c)), color=colors["white"].darker(0.1))
    old_grid += line(((c, -size), (c, size)), color=colors["white"].darker(0.1))
new_grid = Graphics()
for i in xrange(-2*size, 2*size + 1):
    if i == 0:
        new_grid += line((-2*size*u1, 2*size*u1), color=color1, thickness=2)
        new_grid += text("$R$", u1/abs(u1[0])*size*1.1, color=color1, fontsize=16)
        new_grid += line((-2*size*u2, 2*size*u2), color=color2, thickness=2)
        new_grid += text("$S$", u2/abs(u2[0])*size*1.1, color=color2, fontsize=16)
    else:
        new_grid += line((u2*i - 2*size*u1, u2*i + 2*size*u1), color=color1.lighter(0.7))
        new_grid += line((u1*i - 2*size*u2, u1*i + 2*size*u2), color=color2.lighter(0.7))
@interact
def newcoordinates(x=(-size, size, 1), y=(-size, size, 1),
        show_old_grid=checkbox(False, label=u"Original x-y \u201ccoordinate grid\u201d"),
        show_vectors=checkbox(True, label="Eigenvectors"),
        show_new_grid=checkbox(False, label=u"Eigenvector \u201ccoordinate grid\u201d"),
        show_solution=checkbox(False, label="Solution to point along eigenvectors"),
    ):
    p = Graphics()
    if show_old_grid:
        p += old_grid
    if show_vectors:
        p += eigenvectors
    if show_new_grid:
        p += new_grid
    if show_solution:
        r, s = T.inverse()*vector([x, y])
        p += plot(r*u1, color=color1.darker(0.4))
        p += plot(s*u2, color=color2.darker(0.4), start=r*u1)
        print("r = {}, s = {}. That is, ({}, {}) = {} u1  +  {} u2".format(r, s, x, y, r, s))
    p += point((x, y), color="black", size=40)
    show(p, xmin=-size, xmax=size, ymin=-size, ymax=size, figsize=6, aspect_ratio=1, axes_labels=("$X$", "$Y$"))
︡75c63ef0-3aba-444d-821b-021bb6dbe685︡{"interact":{"controls":[{"animate":true,"control_type":"slider","default":0,"display_value":true,"label":"x","vals":["-8","-7","-6","-5","-4","-3","-2","-1","0","1","2","3","4","5","6","7","8"],"var":"x","width":null},{"animate":true,"control_type":"slider","default":0,"display_value":true,"label":"y","vals":["-8","-7","-6","-5","-4","-3","-2","-1","0","1","2","3","4","5","6","7","8"],"var":"y","width":null},{"control_type":"checkbox","default":false,"label":"Original x-y “coordinate grid”","readonly":false,"var":"show_old_grid"},{"control_type":"checkbox","default":true,"label":"Eigenvectors","readonly":false,"var":"show_vectors"},{"control_type":"checkbox","default":false,"label":"Eigenvector “coordinate grid”","readonly":false,"var":"show_new_grid"},{"control_type":"checkbox","default":false,"label":"Solution to point along eigenvectors","readonly":false,"var":"show_solution"}],"flicker":false,"id":"325d1124-989d-4f30-b2f0-44dffc838ea2","layout":[[["x",12,null]],[["y",12,null]],[["show_old_grid",12,null]],[["show_vectors",12,null]],[["show_new_grid",12,null]],[["show_solution",12,null]],[["",12,null]]],"style":"None"}}︡{"done":true}︡
︠07e4d3a6-81c4-49aa-925b-bbbf6ec249da︠










︡14a75a14-d9ec-4b7e-8821-7dacddbe8440︡
︠b27cba5c-b2cb-4633-badd-c9694e5348e9i︠
%hide
%md
# Putting it all together...

### The black bear population model again, using the new “$R, S$-coordinates”

Note that in the black bear population model, the eigenvectors of the matrix were exactly the same vectors $\vec{u}_1$ and $\vec{u}_2$ that we used just now to define the $R, S$-coordinate system. The eigenvalue corresponding to $\vec{u}_1$ was 1.15, and the eigenvalue corresponding to $\vec{u}_2$ was 0.4.

First, a reminder of what that means about exponential growth/decay along the eigenvector lines:
︡0835d4c9-fe62-4283-a47a-fc81a47c922b︡{"hide":"input"}︡{"md":"# Putting it all together...\n\n### The black bear population model again, using the new “$R, S$-coordinates”\n\nNote that in the black bear population model, the eigenvectors of the matrix were exactly the same vectors $\\vec{u}_1$ and $\\vec{u}_2$ that we used just now to define the $R, S$-coordinate system. The eigenvalue corresponding to $\\vec{u}_1$ was 1.15, and the eigenvalue corresponding to $\\vec{u}_2$ was 0.4. \n\nFirst, a reminder of what that means about exponential growth/decay along the eigenvector lines: "}︡{"done":true}︡
︠edac82ec-4ab4-4429-b915-21b718c1393e︠
M = matrix(RDF, [
    [0.65, 0.5],
    [0.25, 0.9],
])
u1 = vector([1, 1])
u2 = vector([-2, 1])
T = column_matrix([u1, u2])
color1 = colors["red"].darker(0.2)
color2 = colors["green"].darker(0.2)
size = 12
eigenvectors = plot(u1, color=color1, arrowsize=3) + plot(u2, color=color2, arrowsize=3)
old_grid = Graphics()
for c in xrange(-size + 1, size):
    old_grid += line(((-size, c), (size, c)), color=colors["white"].darker(0.1))
    old_grid += line(((c, -size), (c, size)), color=colors["white"].darker(0.1))
new_grid = Graphics()
for i in xrange(-2*size, 2*size + 1):
    if i == 0:
        new_grid += line((-2*size*u1, 2*size*u1), color=color1, thickness=2)
        new_grid += text("$R$", u1/abs(u1[0])*size*1.1, color=color1, fontsize=16)
        new_grid += line((-2*size*u2, 2*size*u2), color=color2, thickness=2)
        new_grid += text("$S$", u2/abs(u2[0])*size*1.1, color=color2, fontsize=16)
    else:
        new_grid += line((u2*i - 2*size*u1, u2*i + 2*size*u1), color=color1.lighter(0.7))
        new_grid += line((u1*i - 2*size*u2, u1*i + 2*size*u2), color=color2.lighter(0.7))
ics = {
    tuple(u1): color1.darker(0.6),
    tuple(u2/abs(u2[0])*size): color2.darker(0.6),
}

@interact
def newcoordinates(iterations=(0, 20, 1),
        plotjoined=checkbox(True, label="Connect dots"),
        show_old_grid=checkbox(True, label=u"Original x-y \u201ccoordinate grid\u201d"),
        show_vectors=checkbox(False, label="Eigenvectors"),
        show_new_grid=checkbox(False, label=u"Eigenvector \u201ccoordinate grid\u201d"),
    ):
    p = Graphics()
    if show_old_grid:
        p += old_grid
    if show_vectors:
        p += eigenvectors
    if show_new_grid:
        p += new_grid
    for state, color in ics.items():
        states = [vector(state)]
        for i in xrange(iterations):
            states.append(M*states[-1])
        p += list_plot(states, plotjoined=False, color=color, size=50)
        if plotjoined:
            p += list_plot(states, plotjoined=True, color=color)
    show(M)
    show(p, xmin=-size, xmax=size, ymin=-size, ymax=size, figsize=10, aspect_ratio=1, axes_labels=("$X$", "$Y$"))
︡79f99791-2c48-4c03-8dc2-860bae1fadef︡{"interact":{"controls":[{"animate":true,"control_type":"slider","default":0,"display_value":true,"label":"iterations","vals":["0","1","2","3","4","5","6","7","8","9","10","11","12","13","14","15","16","17","18","19","20"],"var":"iterations","width":null},{"control_type":"checkbox","default":true,"label":"Connect dots","readonly":false,"var":"plotjoined"},{"control_type":"checkbox","default":true,"label":"Original x-y “coordinate grid”","readonly":false,"var":"show_old_grid"},{"control_type":"checkbox","default":false,"label":"Eigenvectors","readonly":false,"var":"show_vectors"},{"control_type":"checkbox","default":false,"label":"Eigenvector “coordinate grid”","readonly":false,"var":"show_new_grid"}],"flicker":false,"id":"13f12343-4400-4baf-998b-dd95eee8b568","layout":[[["iterations",12,null]],[["plotjoined",12,null]],[["show_old_grid",12,null]],[["show_vectors",12,null]],[["show_new_grid",12,null]],[["",12,null]]],"style":"None"}}︡{"done":true}︡
︠36fdd362-cd72-4f33-8c38-c7c726d41de3i︠
%hide
%md
Now, explore how those other trajectories (that don't just start on the eigenvector lines) happen:

Be sure to click the “show projections” checkbox after simulating one of the trajectories.

You can also uncomment other initial conditions (find “ics” in the code) to see other trajectories.
︡8ea0d925-7ba0-4f10-b6ad-85c23a9a013a︡{"hide":"input"}︡{"md":"Now, explore how those other trajectories (that don't just start on the eigenvector lines) happen: \n\nBe sure to click the “show projections” checkbox after simulating one of the trajectories. \n\nYou can also uncomment other initial conditions (find “ics” in the code) to see other trajectories. \n"}︡{"done":true}︡
︠498d942c-6576-41c1-adc7-1b57509d35d4︠
M = matrix(RDF, [
    [0.65, 0.5],
    [0.25, 0.9],
])
u1 = vector([1, 1])
u2 = vector([-2, 1])
T = column_matrix([u1, u2])
color1 = colors["red"].darker(0.2)
color2 = colors["green"].darker(0.2)
size = 12
eigenvectors = plot(u1, color=color1, arrowsize=3) + plot(u2, color=color2, arrowsize=3)
old_grid = Graphics()
for c in xrange(-size + 1, size):
    old_grid += line(((-size, c), (size, c)), color=colors["white"].darker(0.1))
    old_grid += line(((c, -size), (c, size)), color=colors["white"].darker(0.1))
new_grid = Graphics()
for i in xrange(-2*size, 2*size + 1):
    if i == 0:
        new_grid += line((-2*size*u1, 2*size*u1), color=color1, thickness=2)
        new_grid += text("$R$", u1/abs(u1[0])*size*1.1, color=color1, fontsize=16)
        new_grid += line((-2*size*u2, 2*size*u2), color=color2, thickness=2)
        new_grid += text("$S$", u2/abs(u2[0])*size*1.1, color=color2, fontsize=16)
    else:
        new_grid += line((u2*i - 2*size*u1, u2*i + 2*size*u1), color=color1.lighter(0.7))
        new_grid += line((u1*i - 2*size*u2, u1*i + 2*size*u2), color=color2.lighter(0.7))
ics = {
    ( 2,  5): "blue",
    #( 7,  4): "purple",
    #( 0,  3): "gold",
    #( 9,  0): "fuchsia",
}

@interact
def newcoordinates(iterations=(0, 20, 1),
        plotjoined=checkbox(True, label="Connect dots"),
        show_old_grid=checkbox(False, label=u"Original x-y \u201ccoordinate grid\u201d"),
        show_vectors=checkbox(False, label="Eigenvectors"),
        show_new_grid=checkbox(True, label=u"Eigenvector \u201ccoordinate grid\u201d"),
        show_projections=checkbox(False, label="Show projections onto eigenvector axes"),
        only=checkbox(False, label="Show only those projections"),
    ):
    p = Graphics()
    if show_old_grid:
        p += old_grid
    if show_vectors:
        p += eigenvectors
    if show_new_grid:
        p += new_grid
    if show_projections:
        all_ics = {}
        for state, color in ics.items():
            R, S = T.inverse()*vector(state)
            all_ics[tuple(R*u1)] = color1.darker(0.6)
            all_ics[tuple(S*u2)] = color2.darker(0.6)
            if not only:
                all_ics[state] = color
    else:
        all_ics = ics
    for state, color in all_ics.items():
        states = [vector(state)]
        for i in xrange(iterations):
            states.append(M*states[-1])
        p += list_plot(states, plotjoined=False, color=color, size=50)
        if plotjoined:
            p += list_plot(states, plotjoined=True, color=color)
    show(M)
    show(p, xmin=-size, xmax=size, ymin=-size, ymax=size, figsize=10, aspect_ratio=1, axes_labels=("$X$", "$Y$"))
︡458935aa-5fd4-4d99-b868-e15967e3182c︡{"interact":{"controls":[{"animate":true,"control_type":"slider","default":0,"display_value":true,"label":"iterations","vals":["0","1","2","3","4","5","6","7","8","9","10","11","12","13","14","15","16","17","18","19","20"],"var":"iterations","width":null},{"control_type":"checkbox","default":true,"label":"Connect dots","readonly":false,"var":"plotjoined"},{"control_type":"checkbox","default":false,"label":"Original x-y “coordinate grid”","readonly":false,"var":"show_old_grid"},{"control_type":"checkbox","default":false,"label":"Eigenvectors","readonly":false,"var":"show_vectors"},{"control_type":"checkbox","default":true,"label":"Eigenvector “coordinate grid”","readonly":false,"var":"show_new_grid"},{"control_type":"checkbox","default":false,"label":"Show projections onto eigenvector axes","readonly":false,"var":"show_projections"},{"control_type":"checkbox","default":false,"label":"Show only those projections","readonly":false,"var":"only"}],"flicker":false,"id":"5f140eb3-f3b1-4a1e-ab01-31f1a73845ef","layout":[[["iterations",12,null]],[["plotjoined",12,null]],[["show_old_grid",12,null]],[["show_vectors",12,null]],[["show_new_grid",12,null]],[["show_projections",12,null]],[["only",12,null]],[["",12,null]]],"style":"None"}}︡{"done":true}︡
︠234cd332-aea2-4c46-b7f3-fda2f3ab4395i︠
%hide
%md
### The above simulation(s) show the point of diagonalizing a matrix:

When you treat the system entirely in the $R, S$-coordinate system defined by the eigenvectors, the matrix becomes diagonal, which means that the behavior of the entire system can be understood just like for a diagonal matrix. The only difference is that instead of thinking in terms of $X, Y$-coordinates, and all the growth and decay happening along the $X$ and $Y$ axes, now everything in the state space must be handled in the $R, S$-coordinate system, and all the exponential growth/decay happens along the axes of the that coordinate system... which are the eigenvector lines.

In particular, this means that everything we said above about the dominant eigenvector, and the types of equilibrium point at the origin, all carry over to arbitrary matrices, not just diagonal ones.
︡f420dc92-45d6-4424-9e6b-783e4fb35095︡{"hide":"input"}︡{"md":"### The above simulation(s) show the point of diagonalizing a matrix: \n\nWhen you treat the system entirely in the $R, S$-coordinate system defined by the eigenvectors, the matrix becomes diagonal, which means that the behavior of the entire system can be understood just like for a diagonal matrix. The only difference is that instead of thinking in terms of $X, Y$-coordinates, and all the growth and decay happening along the $X$ and $Y$ axes, now everything in the state space must be handled in the $R, S$-coordinate system, and all the exponential growth/decay happens along the axes of the that coordinate system... which are the eigenvector lines. \n\nIn particular, this means that everything we said above about the dominant eigenvector, and the types of equilibrium point at the origin, all carry over to arbitrary matrices, not just diagonal ones. "}︡{"done":true}︡
︠db453d43-1a78-429a-ab7a-b3b590d2f0eb︠










︡07d7acb9-7ef9-4bb6-93d0-e951997a5402︡
︠d28348bf-47f2-48b9-b994-3dd1cb927d1fi︠
%hide
%md
# Examples of diagonalizing

### Black bear population model:
︡c1fa1ac0-a76c-40b7-bc6d-7e0ac4cf98b3︡{"hide":"input"}︡{"md":"# Examples of diagonalizing\n\n### Black bear population model:"}︡{"done":true}︡
︠7ceed3e3-8526-42fd-8775-0af45e931212︠
M = matrix(RDF, [
    [0.65, 0.5],
    [0.25, 0.9],
])
show("$M = $", M)
M.eigenvalues()
u1 = vector([1, 1])
u2 = vector([-2, 1])
T = column_matrix([u1, u2])
show("$T = $", T)
print("\n\n")

D = T.inverse()*M*T
show("$D = $", D.round(6))
︡d6796d1c-6742-4e17-992b-2c450fe71fad︡{"html":"<div align='center'>$M = $ $\\displaystyle \\left(\\begin{array}{rr}\n0.65 &amp; 0.5 \\\\\n0.25 &amp; 0.9\n\\end{array}\\right)$</div>"}︡{"stdout":"[0.4, 1.15]\n"}︡{"html":"<div align='center'>$T = $ $\\displaystyle \\left(\\begin{array}{rr}\n1 &amp; -2 \\\\\n1 &amp; 1\n\\end{array}\\right)$</div>"}︡{"stdout":"\n\n\n"}︡{"html":"<div align='center'>$D = $ $\\displaystyle \\left(\\begin{array}{rr}\n1.15 &amp; 0.0 \\\\\n-0.0 &amp; 0.4\n\\end{array}\\right)$</div>"}︡{"done":true}︡
︠a41aa300-2c8a-41ca-a1cf-bd96347fa083i︠
%hide
%md
### Mares and fillies model from Midterm 2:
︡4b14d6cd-6bce-4518-96d0-7a2d89427af2︡{"hide":"input"}︡{"md":"### Mares and fillies model from Midterm 2:"}︡{"done":true}︡
︠86f4bbcc-1f64-4f0a-b2b9-a5dfc1d7f61e︠
M = matrix(RDF, [
    [0.35, 0.25],
    [0.45, 0.95],
])
show("$M = $", M)
[round(eigenvalue, 5) for eigenvalue in M.eigenvalues()]
u1 = vector([1, 3])
u2 = vector([5, -3])
T = column_matrix([u1, u2])
show("$T = $", T)
print("\n\n")

D = T.inverse()*M*T
show("$D = $", D.round(6))
︡7c86e255-f262-4b27-add9-f6ab90134e10︡{"html":"<div align='center'>$M = $ $\\displaystyle \\left(\\begin{array}{rr}\n0.35 &amp; 0.25 \\\\\n0.45 &amp; 0.95\n\\end{array}\\right)$</div>"}︡{"stdout":"[0.2, 1.1]\n"}︡{"html":"<div align='center'>$T = $ $\\displaystyle \\left(\\begin{array}{rr}\n1 &amp; 5 \\\\\n3 &amp; -3\n\\end{array}\\right)$</div>"}︡{"stdout":"\n\n\n"}︡{"html":"<div align='center'>$D = $ $\\displaystyle \\left(\\begin{array}{rr}\n1.1 &amp; 0.0 \\\\\n0.0 &amp; 0.2\n\\end{array}\\right)$</div>"}︡{"done":true}︡
︠999e4fe0-0077-4de8-8b94-04f1e4cd7d77i︠
%hide
%md
### Lionfish model from lab:

This one is 3-dimensional, but diagonalization still works the same way.
︡f020d51e-a493-49d2-be13-2d55d883ba64︡{"hide":"input"}︡{"md":"### Lionfish model from lab:\n\nThis one is 3-dimensional, but diagonalization still works the same way. "}︡{"done":true}︡
︠51655e33-b931-4587-b7d6-238f13a1e5ff︠
M = matrix(RDF, [
        [0,       0,     35315    ],
        [0.00003, 0.777,     0    ],
        [0,       0.071,     0.949],
    ])
show("$M = $", M)
M.eigenvalues()
T = column_matrix([evec[0] for eval, evec, mult in M.eigenvectors_right()])
show("$T = $", T.round(8))
print("\n\n")

D = T.inverse()*M*T
show("$D = $", D.round(4))
︡2454dcb1-f912-4247-84af-18a4055e3086︡{"html":"<div align='center'>$M = $ $\\displaystyle \\left(\\begin{array}{rrr}\n0.0 &amp; 0.0 &amp; 35315.0 \\\\\n3 \\times 10^{-05} &amp; 0.777 &amp; 0.0 \\\\\n0.0 &amp; 0.071 &amp; 0.949\n\\end{array}\\right)$</div>"}︡{"stdout":"[0.15026156162015658, 0.44126018150798385, 1.1344782568718599]\n"}︡{"html":"<div align='center'>$T = $ $\\displaystyle \\left(\\begin{array}{rrr}\n1.0 &amp; -1.0 &amp; 1.0 \\\\\n-4.787 \\times 10^{-05} &amp; 8.935 \\times 10^{-05} &amp; 8.392 \\times 10^{-05} \\\\\n4.25 \\times 10^{-06} &amp; -1.249 \\times 10^{-05} &amp; 3.212 \\times 10^{-05}\n\\end{array}\\right)$</div>"}︡{"stdout":"\n\n\n"}︡{"html":"<div align='center'>$D = $ $\\displaystyle \\left(\\begin{array}{rrr}\n0.1503 &amp; -0.0 &amp; -0.0 \\\\\n0.0 &amp; 0.4413 &amp; -0.0 \\\\\n-0.0 &amp; -0.0 &amp; 1.1345\n\\end{array}\\right)$</div>"}︡{"done":true}︡
︠dc3e4a04-1e08-4350-888e-3d81ba0a2c40i︠
%hide
%md
### What about complex eigenvalues?

Here is a 3-dimensional model that has a pair of complex eigenvalues. Recall that complex eigenvalues give rise to rotating/oscillating behavior (usually combined with exponential growth (spiralling outward) or exponential decay (spiralling inward), depending on whether the absolute value of those eigenvalues is greater than or less than 1).
︡2061f7e4-bea9-4025-bd36-6f0c259c0783︡{"hide":"input"}︡{"md":"### What about complex eigenvalues? \n\nHere is a 3-dimensional model that has a pair of complex eigenvalues. Recall that complex eigenvalues give rise to rotating/oscillating behavior (usually combined with exponential growth (spiralling outward) or exponential decay (spiralling inward), depending on whether the absolute value of those eigenvalues is greater than or less than 1). "}︡{"done":true}︡
︠00e26700-ed54-411c-8657-07ea2cfe4cd1︠
M = matrix(RDF, [
        [ 0.46,  0.64,  0.08],
        [-0.08,  1.18, -0.64],
        [-0.72,  0.72,  0.54],
    ])
show("$M = $", M)
print("\n")
for eigenvalue in M.eigenvalues():
    show(eigenvalue)
︡ccd56c62-8dc1-45f0-957d-abad17e41ef6︡{"html":"<div align='center'>$M = $ $\\displaystyle \\left(\\begin{array}{rrr}\n0.46 &amp; 0.64 &amp; 0.08 \\\\\n-0.08 &amp; 1.18 &amp; -0.64 \\\\\n-0.72 &amp; 0.72 &amp; 0.54\n\\end{array}\\right)$</div>"}︡{"stdout":"\n\n"}︡{"html":"<div align='center'>$\\displaystyle 0.54 + 0.72i$</div>"}︡{"html":"<div align='center'>$\\displaystyle 0.54 - 0.72i$</div>"}︡{"html":"<div align='center'>$\\displaystyle 1.1$</div>"}︡{"done":true}︡
︠56ac0967-97ca-4a5f-8b63-0b50acf3512di︠
%hide
%md
### One way of handling this: just use the complex eigenvectors

This diagonalizes the matrix perfectly, as usual. But what does it really tell us about where the trajectories will go and how they will behave?
︡2bbd57be-94e9-4a7d-b31c-823228af3ec2︡{"hide":"input"}︡{"md":"### One way of handling this: just use the complex eigenvectors\n\nThis diagonalizes the matrix perfectly, as usual. But what does it really tell us about where the trajectories will go and how they will behave? "}︡{"done":true}︡
︠16d47c99-cec4-4c45-8735-f3fe8eac12d5︠
T = column_matrix([evec[0]*sqrt(2).n() for eval, evec, mult in M.eigenvectors_right()])
show("$T = $", T.round(8))
print("\n\n")

D = T.inverse()*M*T
show("$D = $", D.round(4))
︡5364712d-4129-40bc-83db-5e1f195a204a︡{"html":"<div align='center'>$T = $ $\\displaystyle \\left(\\begin{array}{rrr}\n0.5 - 0.5i &amp; 0.5 + 0.5i &amp; -1.0 \\\\\n0.5 + 0.5i &amp; 0.5 - 0.5i &amp; -1.0 \\\\\n1.0 &amp; 1.0 &amp; 0.0\n\\end{array}\\right)$</div>"}︡{"stdout":"\n\n\n"}︡{"html":"<div align='center'>$D = $ $\\displaystyle \\left(\\begin{array}{rrr}\n0.54 + 0.72i &amp; 0.0 &amp; 0.0 \\\\\n0.0 &amp; 0.54 - 0.72i &amp; 0.0 \\\\\n0.0 &amp; 0.0 &amp; 1.1\n\\end{array}\\right)$</div>"}︡{"done":true}︡
︠1841150f-c718-4d38-8123-07580edb0080i︠
%hide
%md
### Perhaps a better way:

Take one of the complex eigenvectors (it doesn't matter which one, since they're complex conjugates of each other) and separate its real and imaginary parts into two separate vectors. Use these to form the columns of the transformation matrix $T$. Now look what happens...
︡77794345-1e88-4929-b0bb-d5ee5e8f4f08︡{"hide":"input"}︡{"md":"### Perhaps a better way:\n\nTake one of the complex eigenvectors (it doesn't matter which one, since they're complex conjugates of each other) and separate its real and imaginary parts into two separate vectors. Use these to form the columns of the transformation matrix $T$. Now look what happens..."}︡{"done":true}︡
︠c1579099-105d-4a30-9d4a-9fc6de5fc65d︠
T = transformation_matrix(M)*sqrt(2).n()
show("$T = $", T.round(8))
print("\n\n")

D = T.inverse()*M*T
show("$D = $", D.round(4))
︡d76234ce-f13e-44fd-978f-6c925c16a51e︡{"html":"<div align='center'>$T = $ $\\displaystyle \\left(\\begin{array}{rrr}\n0.5 &amp; -0.5 &amp; -1.0 \\\\\n0.5 &amp; 0.5 &amp; -1.0 \\\\\n1.0 &amp; 0.0 &amp; -0.0\n\\end{array}\\right)$</div>"}︡{"stdout":"\n\n\n"}︡{"html":"<div align='center'>$D = $ $\\displaystyle \\left(\\begin{array}{rrr}\n0.54 &amp; 0.72 &amp; 0.0 \\\\\n-0.72 &amp; 0.54 &amp; 0.0 \\\\\n0.0 &amp; 0.0 &amp; 1.1\n\\end{array}\\right)$</div>"}︡{"done":true}︡
︠ac095e70-e014-4511-932c-1d9843bcee5eis︠
%hide
%md
This matrix is not diagonal, but it is close: it is called a “block diagonal” matrix. The 2×2 “block” that appears in the upper left part of the matrix has the form
$$ \begin{bmatrix} a & b \\ -b & a \end{bmatrix} $$
where the $a$ and $b$ are exactly the real and imaginary parts of the complex eigenvalues of our original matrix! The eigenvalues of this 2x2 block, by itself, are also exactly $a + bi$ and $a - bi$, the same as those of our original matrix.

A 2×2 matrix of this form, $\begin{bmatrix} a & b \\ -b & a \end{bmatrix}$, as a discrete-time linear model, will have trajectories that rotate/oscillate around the origin, as they exponentially grow (spiral outward) or exponentially decay (spiral inward). Once again, whether they grow or decay, and how fast, depends on the value of $\sqrt{a^2 + b^2}$... which is just the absolute value of the two complex eigenvalues. Note that in our example,

$$ \sqrt{a^2 + b^2} = \sqrt{0.54^2 + 0.72^2} = 0.9 < 1 $$

so this 2×2 block will cause spiralling inward.

What does all of the above mean for the behavior of our original 3×3 matrix $M$? The matrix $T$ given above still works as a transformation matrix that converts between coordinate systems, in the usual way: the columns of $T$ are a basis (of $\RR^3$), and this basis defines a new coordinate system, which we might call $Q, R, S$-coordinates. What the block-diagonal form of $D$ means is that *in the $Q, R$-plane in this coordinate system*, trajectories will spiral inward. Along the $S$ axis, trajectories will exponentially grow (at 10% per iteration). Everywhere else in the state space, the behavior will be a combination of these two features, just as we've seen in all the previous examples. Here is a 3-D rendering of this, showing the $Q, R$-plane (in green), the $S$-axis (in red), and a typical trajectory that starts close to the $Q, R$-plane. This is one version of a 3-dimensional saddle point, one that includes rotating/spiralling behavior.
︡82762053-6e54-4d28-be67-4f3ad22bf0b2︡{"hide":"input"}︡{"md":"This matrix is not diagonal, but it is close: it is called a “block diagonal” matrix. The 2×2 “block” that appears in the upper left part of the matrix has the form\n$$ \\begin{bmatrix} a & b \\\\ -b & a \\end{bmatrix} $$\nwhere the $a$ and $b$ are exactly the real and imaginary parts of the complex eigenvalues of our original matrix! The eigenvalues of this 2x2 block, by itself, are also exactly $a + bi$ and $a - bi$, the same as those of our original matrix.\n\nA 2×2 matrix of this form, $\\begin{bmatrix} a & b \\\\ -b & a \\end{bmatrix}$, as a discrete-time linear model, will have trajectories that rotate/oscillate around the origin, as they exponentially grow (spiral outward) or exponentially decay (spiral inward). Once again, whether they grow or decay, and how fast, depends on the value of $\\sqrt{a^2 + b^2}$... which is just the absolute value of the two complex eigenvalues. Note that in our example,\n\n$$ \\sqrt{a^2 + b^2} = \\sqrt{0.54^2 + 0.72^2} = 0.9 < 1 $$\n\nso this 2×2 block will cause spiralling inward.\n\nWhat does all of the above mean for the behavior of our original 3×3 matrix $M$? The matrix $T$ given above still works as a transformation matrix that converts between coordinate systems, in the usual way: the columns of $T$ are a basis (of $\\RR^3$), and this basis defines a new coordinate system, which we might call $Q, R, S$-coordinates. What the block-diagonal form of $D$ means is that *in the $Q, R$-plane in this coordinate system*, trajectories will spiral inward. Along the $S$ axis, trajectories will exponentially grow (at 10% per iteration). Everywhere else in the state space, the behavior will be a combination of these two features, just as we've seen in all the previous examples. Here is a 3-D rendering of this, showing the $Q, R$-plane (in green), the $S$-axis (in red), and a typical trajectory that starts close to the $Q, R$-plane. This is one version of a 3-dimensional saddle point, one that includes rotating/spiralling behavior.\n"}︡{"done":true}︡
︠d1787885-4308-403e-9b56-77975dd4df8d︠
ics = {(3, -3, 0.1): 46}
plot_eigenspaces3d(M, ics).show(frame=False)
︡a19a23df-c57e-453b-bf02-4aa6b40e0b2a︡{"file":{"filename":"a113c6ec-81b8-4f75-b4d1-527d9907f16e.sage3d","uuid":"a113c6ec-81b8-4f75-b4d1-527d9907f16e"}}︡{"done":true}︡︡
︠d6660df4-3819-49d4-b495-b20b17faef84︠









